"""
-------------------------------------------------
   File Name：   RSAUtil
   Description:  PYTHON-RSA 加密/解密、加签/验签
   Author:       CANGMU
   date：        2024/5/27
-------------------------------------------------
"""
import base64
import binascii
import hashlib
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.serialization import pkcs12
import rsa

class RSAUtil:
    @staticmethod
    def load_key(cert_file_path, password=None):
        with open(cert_file_path, 'rb') as f:
            cert_data = f.read()

        if cert_file_path.endswith(('.pfx', '.p12')):
            private_key, certificate, additional_certificates = pkcs12.load_key_and_certificates(
                cert_data,
                password.encode() if password else None,
                backend=default_backend()
            )
            pri_key_pem = private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption()
            ).decode()
            return rsa.PrivateKey._load_pkcs1_pem(pri_key_pem)
        else:
            certificate = x509.load_pem_x509_certificate(cert_data, backend=default_backend())
            pub_key_pem = certificate.public_key().public_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PublicFormat.SubjectPublicKeyInfo
            ).decode()
            return rsa.PublicKey.load_pkcs1_openssl_pem(pub_key_pem)

    @staticmethod
    def _chunk_data(data, chunk_size):
        while data:
            yield data[:chunk_size]
            data = data[chunk_size:]

    @staticmethod
    def pri_encrypt(encrypt_text, pfx_path, pfxpass):
        key = RSAUtil.load_key(pfx_path, pfxpass)
        block_size = key.n.bit_length() // 8 - 11
        key_length = key.n.bit_length() // 8
        encrypt_text = base64.b64encode(encrypt_text.encode('utf-8'))
        out_data = bytes()

        for chunk in RSAUtil._chunk_data(encrypt_text, block_size):
            chunk = rsa.pkcs1._pad_for_signing(chunk, key_length)
            num = rsa.transform.bytes2int(chunk)
            encrypted = rsa.core.encrypt_int(num, key.d, key.n)
            encrypted_bytes = rsa.transform.int2bytes(encrypted)

            # 确保加密数据填充前导零以匹配密钥长度
            pad_length = key_length - len(encrypted_bytes)
            padded_data = b'\x00' * pad_length + encrypted_bytes

            out_data += padded_data

        return out_data.hex()

    @staticmethod
    def pub_decrypt(decrypt_text, cer_path):
        key = RSAUtil.load_key(cer_path)
        block_size = key.n.bit_length() // 8
        decrypt_text = binascii.unhexlify(decrypt_text)
        out_data = bytes()

        for chunk in RSAUtil._chunk_data(decrypt_text, block_size):
            num = rsa.transform.bytes2int(chunk)
            decrypted = rsa.core.decrypt_int(num, key.e, key.n)
            out_data += rsa.transform.int2bytes(decrypted)

        return base64.b64decode(out_data).decode('utf-8')

    @staticmethod
    def pub_encrypt(encrypt_text, cer_path):
        key = RSAUtil.load_key(cer_path)
        block_size = key.n.bit_length() // 8 - 11
        encrypt_text = base64.b64encode(encrypt_text.encode('utf-8'))
        out_data = bytes()

        for chunk in RSAUtil._chunk_data(encrypt_text, block_size):
            encrypted = rsa.pkcs1.encrypt(chunk, key)
            out_data += encrypted

        return out_data.hex()

    @staticmethod
    def pri_decrypt(decrypt_text, pfx_path, pfxpass):
        key = RSAUtil.load_key(pfx_path, pfxpass)
        block_size = key.n.bit_length() // 8
        decrypt_text = binascii.unhexlify(decrypt_text)
        out_data = bytes()

        for chunk in RSAUtil._chunk_data(decrypt_text, block_size):
            decrypted = rsa.pkcs1.decrypt(chunk, key)
            out_data += decrypted

        return base64.b64decode(out_data).decode('utf-8')

    @staticmethod
    def _get_algorithm_name(hash_algorithm):
        algorithm_name = {
            'sha256': 'SHA-256',
            'sha1': 'SHA-1'
        }.get(hash_algorithm().name, 'Unsupported Algorithm')
        return algorithm_name

    @staticmethod
    def _sign_data(sign_text, hash_algorithm, pfx_path, pfxpass):
        key = RSAUtil.load_key(pfx_path, pfxpass)
        hash_value = hash_algorithm(sign_text.encode('utf-8')).digest()
        algorithm_name = RSAUtil._get_algorithm_name(hash_algorithm)
        signature = rsa.sign(hash_value, key, algorithm_name)
        return signature.hex()

    @staticmethod
    def _verify_signature(data, sign_text, hash_algorithm, cer_path):
        key = RSAUtil.load_key(cer_path)
        hash_value = hash_algorithm(data.encode('utf-8')).digest()
        try:
            rsa.verify(hash_value, binascii.unhexlify(sign_text), key)
            print("Verification successful")
        except rsa.VerificationError:
            print("Verification failed")

    @staticmethod
    def pri_sha1_sign(sign_text, pfx_path, pfxpass):
        return RSAUtil._sign_data(sign_text, hashlib.sha1, pfx_path, pfxpass)

    @staticmethod
    def pub_sha1_verify(data, sign_text, cer_path):
        RSAUtil._verify_signature(data, sign_text, hashlib.sha1, cer_path)

    @staticmethod
    def pri_sha256_sign(sign_text, pfx_path, pfxpass):
        return RSAUtil._sign_data(sign_text, hashlib.sha256, pfx_path, pfxpass)

    @staticmethod
    def pub_sha256_verify(data, sign_text, cer_path):
        RSAUtil._verify_signature(data, sign_text, hashlib.sha256, cer_path)




# Example usage:
if __name__ == '__main__':
    testData = 'send_time=2018-01-24 13:25:33&msg_id=456795112&version=4.0.0.0&123=100000949&txn_type=03&member_id=100000749&dgtl_envlp=5a9c3ac419735d249e319727c89cfc0ce4a80d6a954980eaf3ea934316a56a121c758b0d13bf3302b877a8dd68619db72b2bd588ccdc9eb7fdb455705be1909df96540009146d7d81c96c0b90578f9344bd3fc00ded94d27c0c8040a83c02114b7a3a4698f830b7d0db60f230a5c3a4b38e7104088f2ee0139a4e765a9d79255&user_id=123'
    pfxpath = 'D:\\GEP\\privatekey.pfx'
    pfxpass = '5181221121000514358_416342'
    cerpath = 'D:\\GEP\\publickey.cer'

    pri_en = RSAUtil.pri_encrypt(testData, pfxpath, pfxpass)
    print("私钥加密内容：{}".format(pri_en))
    print("公钥解密内容：{}".format(RSAUtil.pub_decrypt(pri_en, cerpath)))
